/* eslint-disable react/jsx-handler-names */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import ReactDOM from 'react-dom';
import DomHandler from 'primereact/components/utils/DomHandler';
import ObjectUtils from 'primereact/components/utils/ObjectUtils';
import UniqueComponentId from 'primereact/components/utils/UniqueComponentId';
import classNames from 'classnames';
import Tooltip from 'primereact/tooltip';
import InputText from './InputText';
import AutoCompletePanel from './AutoCompletePanel';
import Button from './Button';

export class AutoComplete extends Component {
	constructor(props) {
		super(props);

		this.onInputChange = this.onInputChange.bind(this);
		this.onInputFocus = this.onInputFocus.bind(this);
		this.onInputBlur = this.onInputBlur.bind(this);
		this.onInputClick = this.onInputClick.bind(this);
		this.onInputKeyDown = this.onInputKeyDown.bind(this);
		this.onDropdownClick = this.onDropdownClick.bind(this);
		this.onMultiContainerClick = this.onMultiContainerClick.bind(this);
		this.onMultiInputFocus = this.onMultiInputFocus.bind(this);
		this.onMultiInputBlur = this.onMultiInputBlur.bind(this);
		this.selectItem = this.selectItem.bind(this);

		// eslint-disable-next-line babel/new-cap
		this.listId = `${UniqueComponentId()}_list`;
	}

	componentDidMount() {
		const { autoFocus, tooltip } = this.props;
		if (autoFocus && this.inputEl) {
			this.inputEl.focus();
		}

		if (tooltip) {
			this.renderTooltip();
		}
	}

	componentDidUpdate(prevProps) {
		const { multiple, suggestions, tooltip, value } = this.props;
		if (this.searching) {
			if (suggestions && suggestions.length) this.showPanel();
			else this.hidePanel();

			this.hideLoader();
		}

		this.searching = false;

		if (this.inputEl && !multiple) {
			this.updateInputField(value);
		}

		if (prevProps.tooltip !== tooltip) {
			if (this.tooltip) this.tooltip.updateContent(tooltip);
			else this.renderTooltip();
		}
	}

	componentWillUnmount() {
		this.unbindDocumentClickListener();

		if (this.tooltip) {
			this.tooltip.destroy();
			this.tooltip = null;
		}
	}

	onInputChange(event) {
		const { delay, minLength, multiple, onClear } = this.props;
		// Cancel the search request if user types within the timeout
		if (this.timeout) {
			clearTimeout(this.timeout);
		}

		const query = event.target.value;
		if (!multiple) {
			this.updateModel(event, query);
		}

		if (query.length === 0) {
			this.hidePanel();
			if (onClear) {
				onClear(event);
			}
		} else {
			if (query.length >= minLength) {
				this.timeout = setTimeout(() => {
					this.search(event, query, 'input');
				}, delay);
			} else {
				this.hidePanel();
			}
		}
	}

	onInputClick(event) {
		const { onClick } = this.props;
		if (this.documentClickListener) {
			this.inputClick = true;
		}

		if (onClick) {
			onClick(event);
		}
	}

	search(event, query, source) {
		const { completeMethod } = this.props;
		// allow empty string but not undefined or null
		if (query === undefined || query === null) {
			return;
		}

		// do not search blank values on input change
		if (source === 'input' && query.trim().length === 0) {
			return;
		}

		if (completeMethod) {
			this.searching = true;
			this.showLoader();
			completeMethod({
				originalEvent: event,
				query,
			});
		}
	}

	selectItem(event, option) {
		const { multiple, onSelect, value } = this.props;
		if (multiple) {
			this.inputEl.value = '';
			if (!this.isSelected(option)) {
				const newValue = value ? [...value, option] : [option];
				this.updateModel(event, newValue);
			}
		} else {
			this.updateInputField(option);
			this.updateModel(event, option);
		}

		if (onSelect) {
			onSelect({
				originalEvent: event,
				value: option,
			});
		}

		this.inputEl.focus();
	}

	updateModel(event, value) {
		const { id, name, onChange } = this.props;
		if (onChange) {
			onChange({
				originalEvent: event,
				value,
				stopPropagation: () => {},
				preventDefault: () => {},
				target: {
					name,
					id,
					value,
				},
			});
		}

		this.ariaSelected = value;
	}

	formatValue(value) {
		const { field, selectedItemTemplate } = this.props;
		if (value) {
			if (selectedItemTemplate) {
				const resolvedFieldData = selectedItemTemplate(value);
				return resolvedFieldData ? resolvedFieldData : value;
			} else if (field) {
				const resolvedFieldData = ObjectUtils.resolveFieldData(value, field);
				return resolvedFieldData !== null && resolvedFieldData !== undefined
					? resolvedFieldData
					: value;
			} else return value;
		} else return '';
	}

	updateInputField(value) {
		const formattedValue = this.formatValue(value);
		this.inputEl.value = formattedValue;
	}

	showPanel() {
		if (this.focus) {
			this.alignPanel();

			if (this.panel && this.panel.element && !this.panel.element.offsetParent) {
				this.panel.element.style.zIndex = String(DomHandler.generateZIndex());
				this.panel.element.style.display = 'block';

				setTimeout(() => {
					if (this.panel && this.panel.element) {
						DomHandler.addClass(this.panel.element, 'p-input-overlay-visible');
						DomHandler.removeClass(this.panel.element, 'p-input-overlay-hidden');
					}
				}, 1);

				this.alignPanel();
				this.bindDocumentClickListener();
			}
		}
	}

	alignPanel() {
		const { appendTo, multiple } = this.props;
		if (this.panel.element.offsetParent) {
			const target = multiple ? this.multiContainer : this.inputEl;

			if (appendTo) {
				this.panel.element.style.minWidth = `${DomHandler.getWidth(target)}px`;
				DomHandler.absolutePosition(this.panel.element, target);
			} else {
				DomHandler.relativePosition(this.panel.element, target);
			}
		}
	}

	hidePanel() {
		DomHandler.addClass(this.panel.element, 'p-input-overlay-hidden');
		DomHandler.removeClass(this.panel.element, 'p-input-overlay-visible');

		setTimeout(() => {
			if (this.panel && this.panel.element) {
				this.panel.element.style.display = 'none';
				DomHandler.removeClass(this.panel.element, 'p-input-overlay-hidden');
			}
		}, 150);

		this.unbindDocumentClickListener();
	}

	onDropdownClick(event) {
		const { dropdownMode, onDropdownClick } = this.props;
		this.inputEl.focus();
		if (this.documentClickListener) {
			this.dropdownClick = true;
		}

		if (dropdownMode === 'blank') this.search(event, '', 'dropdown');
		else if (dropdownMode === 'current') this.search(event, this.inputEl.value, 'dropdown');

		if (onDropdownClick) {
			onDropdownClick({
				originalEvent: event,
				query: this.inputEl.value,
			});
		}
	}

	removeItem(event, index) {
		const { onUnselect, value } = this.props;
		const removedValue = value[index];
		const newValue = value.filter((val, i) => index !== i);
		this.updateModel(event, newValue);

		if (onUnselect) {
			onUnselect({
				originalEvent: event,
				value: removedValue,
			});
		}
	}

	onInputKeyDown(event) {
		const { multiple, onUnselect, suggestions, value } = this.props;
		if (this.isPanelVisible()) {
			const highlightItem = DomHandler.findSingle(this.panel.element, 'li.p-highlight');

			switch (event.which) {
				// down
				case 40:
					if (highlightItem) {
						const nextElement = highlightItem.nextElementSibling;
						if (nextElement) {
							DomHandler.addClass(nextElement, 'p-highlight');
							DomHandler.removeClass(highlightItem, 'p-highlight');
							DomHandler.scrollInView(this.panel.element, nextElement);
						}
					} else {
						DomHandler.addClass(this.panel.element.firstChild.firstChild, 'p-highlight');
					}

					event.preventDefault();
					break;

				// up
				case 38:
					if (highlightItem) {
						const previousElement = highlightItem.previousElementSibling;
						if (previousElement) {
							DomHandler.addClass(previousElement, 'p-highlight');
							DomHandler.removeClass(highlightItem, 'p-highlight');
							DomHandler.scrollInView(this.panel.element, previousElement);
						}
					}

					event.preventDefault();
					break;

				// enter,tab
				case 13:
					if (highlightItem) {
						this.selectItem(event, suggestions[DomHandler.index(highlightItem)]);
						this.hidePanel();
					}

					event.preventDefault();
					break;

				// escape
				case 27:
					this.hidePanel();
					event.preventDefault();
					break;

				// tab
				case 9:
					if (highlightItem) {
						this.selectItem(event, suggestions[DomHandler.index(highlightItem)]);
					}

					this.hidePanel();
					break;

				default:
					break;
			}
		}

		if (multiple) {
			switch (event.which) {
				// backspace
				case 8:
					if (value && value.length && !this.inputEl.value) {
						const removedValue = value[value.length - 1];
						const newValue = value.slice(0, -1);

						if (onUnselect) {
							onUnselect({
								originalEvent: event,
								value: removedValue,
							});
						}

						this.updateModel(event, newValue);
					}
					break;

				default:
					break;
			}
		}
	}

	onInputFocus(event) {
		const { onFocus } = this.props;
		this.focus = true;

		if (onFocus) {
			onFocus(event);
		}

		DomHandler.addClass(this.container, 'p-inputwrapper-focus');
	}

	onInputBlur(event) {
		const { onBlur } = this.props;
		this.focus = false;

		if (onBlur) {
			onBlur(event);
		}

		DomHandler.removeClass(this.container, 'p-inputwrapper-focus');
	}

	onMultiContainerClick(event) {
		const { onClick } = this.props;
		this.inputEl.focus();
		if (this.documentClickListener) {
			this.inputClick = true;
		}

		if (onClick) {
			onClick(event);
		}
	}

	onMultiInputFocus(event) {
		this.onInputFocus(event);
		DomHandler.addClass(this.multiContainer, 'p-focus');
	}

	onMultiInputBlur(event) {
		this.onInputBlur(event);
		DomHandler.removeClass(this.multiContainer, 'p-focus');
	}

	isSelected(val) {
		const { value } = this.props;
		let selected = false;
		if (value && value.length) {
			for (let i = 0; i < value.length; i++) {
				if (ObjectUtils.equals(value[i], val)) {
					selected = true;
					break;
				}
			}
		}

		return selected;
	}

	findOptionIndex(option) {
		let index = -1;
		if (this.suggestions) {
			for (let i = 0; i < this.suggestions.length; i++) {
				if (ObjectUtils.equals(option, this.suggestions[i])) {
					index = i;
					break;
				}
			}
		}

		return index;
	}

	showLoader() {
		this.loader.style.visibility = 'visible';
	}

	hideLoader() {
		this.loader.style.visibility = 'hidden';
	}

	renderTooltip() {
		const { tooltip, tooltipOptions } = this.props;
		this.tooltip = new Tooltip({
			target: this.container,
			content: tooltip,
			options: tooltipOptions,
		});
	}

	renderSimpleAutoComplete() {
		const {
			ariaDescribedBy,
			ariaLabel,
			ariaLabelledBy,
			disabled,
			dropdown,
			inputClassName,
			inputId,
			inputStyle,
			maxlength,
			name,
			onContextMenu,
			onDblClick,
			onKeyPress,
			onKeyUp,
			onMouseDown,
			placeholder,
			readonly,
			required,
			size,
			tabindex,
			type,
			value,
		} = this.props;
		const inputClassNameCalc = classNames('p-autocomplete-input', inputClassName, {
			'p-autocomplete-dd-input': dropdown,
		});

		return (
			<InputText
				ref={el => (this.inputEl = ReactDOM.findDOMNode(el))}
				id={inputId}
				type={type}
				name={name}
				defaultValue={this.formatValue(value)}
				role="searchbox"
				aria-autocomplete="list"
				aria-controls={this.listId}
				aria-describedby={ariaDescribedBy}
				aria-labelledby={ariaLabelledBy}
				aria-label={ariaLabel}
				aria-required={required}
				className={inputClassNameCalc}
				style={inputStyle}
				autoComplete="off"
				readOnly={readonly}
				disabled={disabled}
				placeholder={placeholder}
				size={size}
				maxLength={maxlength}
				tabIndex={tabindex}
				onBlur={this.onInputBlur}
				onFocus={this.onInputFocus}
				onChange={this.onInputChange}
				onMouseDown={onMouseDown}
				onKeyUp={onKeyUp}
				onKeyDown={this.onInputKeyDown}
				onKeyPress={onKeyPress}
				onContextMenu={onContextMenu}
				onClick={this.onInputClick}
				onDoubleClick={onDblClick}
				required={required}
			/>
		);
	}

	renderChips() {
		const { value } = this.props;
		if (value && value.length) {
			return value.map((val, index) => {
				return (
					<li key={`${index}multi-item`} className="p-autocomplete-token p-highlight">
						<span
							className="p-autocomplete-token-icon pi pi-fw pi-times"
							onClick={e => this.removeItem(e, index)}
						/>
						<span className="p-autocomplete-token-label">{this.formatValue(val)}</span>
					</li>
				);
			});
		} else {
			return null;
		}
	}

	renderMultiInput() {
		const {
			ariaDescribedBy,
			ariaLabel,
			ariaLabelledBy,
			disabled,
			inputClassName,
			inputId,
			inputStyle,
			maxlength,
			name,
			onKeyPress,
			onKeyUp,
			placeholder,
			required,
			tabindex,
			type,
		} = this.props;
		return (
			<li className="p-autocomplete-input-token">
				<input
					ref={el => (this.inputEl = el)}
					type={type}
					disabled={disabled}
					placeholder={placeholder}
					role="searchbox"
					aria-autocomplete="list"
					aria-controls={this.listId}
					aria-describedby={ariaDescribedBy}
					aria-labelledby={ariaLabelledBy}
					aria-label={ariaLabel}
					aria-required={required}
					autoComplete="off"
					tabIndex={tabindex}
					onChange={this.onInputChange}
					id={inputId}
					name={name}
					style={inputStyle}
					className={inputClassName}
					maxLength={maxlength}
					onKeyUp={onKeyUp}
					onKeyDown={this.onInputKeyDown}
					onKeyPress={onKeyPress}
					onFocus={this.onMultiInputFocus}
					onBlur={this.onMultiInputBlur}
					required={required}
				/>
			</li>
		);
	}

	renderMultipleAutoComplete() {
		const { disabled, onContextMenu, onDblClick, onMouseDown } = this.props;
		const multiContainerClass = classNames(
			'p-autocomplete-multiple-container p-component p-inputtext',
			{
				'p-disabled': disabled,
			}
		);
		const tokens = this.renderChips();
		const input = this.renderMultiInput();

		return (
			<ul
				ref={el => {
					this.multiContainer = el;
				}}
				className={multiContainerClass}
				onContextMenu={onContextMenu}
				onMouseDown={onMouseDown}
				onClick={this.onMultiContainerClick}
				onDoubleClick={onDblClick}
			>
				{tokens}
				{input}
			</ul>
		);
	}

	renderDropdown() {
		const { disabled, ariaLabel } = this.props;
		return (
			<Button
				type="button"
				icon="pi pi-fw pi-chevron-down"
				className="p-autocomplete-dropdown"
				disabled={disabled}
				onClick={this.onDropdownClick}
				label={`${ariaLabel} naciśnij spację by rozwinąć listę z podpowiedziami`}
			/>
		);
	}

	renderLoader() {
		return (
			<i
				ref={el => (this.loader = el)}
				className="p-autocomplete-loader pi pi-spinner pi-spin"
				style={{ visibility: 'hidden' }}
			/>
		);
	}

	bindDocumentClickListener() {
		if (!this.documentClickListener) {
			this.documentClickListener = event => {
				if (event.which === 3) {
					return;
				}

				if (!this.inputClick && !this.dropdownClick) {
					this.hidePanel();
				}

				this.inputClick = false;
				this.dropdownClick = false;
			};

			document.addEventListener('click', this.documentClickListener);
		}
	}

	unbindDocumentClickListener() {
		if (this.documentClickListener) {
			document.removeEventListener('click', this.documentClickListener);
			this.documentClickListener = null;
		}
	}

	isPanelVisible() {
		return this.panel.element.offsetParent != null;
	}

	render() {
		const {
			appendTo,
			ariaLabel,
			dropdown,
			field,
			id,
			itemTemplate,
			multiple,
			style,
			suggestions,
			value,
		} = this.props;
		let input;
		let dropdownClac;
		const className = classNames('p-autocomplete p-component', className, {
			'p-autocomplete-dd': dropdown,
			'p-autocomplete-multiple': multiple,
			'p-inputwrapper-filled': value,
			'p-inputwrapper-focus': this.focus,
		});
		const loader = this.renderLoader();

		if (multiple) input = this.renderMultipleAutoComplete();
		else input = this.renderSimpleAutoComplete();

		if (dropdown) {
			dropdownClac = this.renderDropdown();
		}

		return (
			<span
				ref={el => (this.container = el)}
				id={id}
				style={style}
				className={className}
				aria-haspopup="listbox"
				aria-expanded={this.panel && this.isPanelVisible()}
				aria-owns={this.listId}
			>
				{input}
				{loader}
				{dropdownClac}
				<AutoCompletePanel
					ref={el => (this.panel = el)}
					suggestions={suggestions}
					field={field}
					listId={this.listId}
					appendTo={appendTo}
					itemTemplate={itemTemplate}
					onItemClick={this.selectItem}
					ariaSelected={this.ariaSelected}
					ariaLabel={ariaLabel}
				/>
			</span>
		);
	}
}

AutoComplete.defaultProps = {
	id: null,
	value: null,
	name: null,
	type: 'text',
	suggestions: null,
	field: null,
	scrollHeight: '200px',
	dropdown: false,
	dropdownMode: 'blank',
	multiple: false,
	minLength: 1,
	delay: 300,
	style: null,
	className: null,
	inputId: null,
	inputStyle: null,
	inputClassName: null,
	placeholder: null,
	readonly: false,
	disabled: false,
	maxlength: null,
	size: null,
	appendTo: null,
	tabindex: null,
	autoFocus: false,
	tooltip: null,
	tooltipOptions: null,
	ariaLabelledBy: null,
	completeMethod: null,
	itemTemplate: null,
	selectedItemTemplate: null,
	onChange: null,
	onFocus: null,
	onBlur: null,
	onSelect: null,
	onUnselect: null,
	onDropdownClick: null,
	onClick: null,
	onDblClick: null,
	onMouseDown: null,
	onKeyUp: null,
	onKeyPress: null,
	onContextMenu: null,
	onClear: null,
};

AutoComplete.propTypes = {
	appendTo: PropTypes.any,
	ariaDescribedBy: PropTypes.string,
	ariaLabel: PropTypes.string,
	ariaLabelledBy: PropTypes.string,
	autoFocus: PropTypes.bool,
	className: PropTypes.string,
	completeMethod: PropTypes.func,
	delay: PropTypes.number,
	disabled: PropTypes.bool,
	dropdown: PropTypes.bool,
	dropdownMode: PropTypes.string,
	field: PropTypes.string,
	id: PropTypes.string,
	inputClassName: PropTypes.string,
	inputId: PropTypes.string,
	inputStyle: PropTypes.object,
	itemTemplate: PropTypes.func,
	maxlength: PropTypes.number,
	minLength: PropTypes.number,
	multiple: PropTypes.bool,
	name: PropTypes.string,
	onBlur: PropTypes.func,
	onChange: PropTypes.func,
	onClear: PropTypes.func,
	onClick: PropTypes.func,
	onContextMenu: PropTypes.func,
	onDblClick: PropTypes.func,
	onDropdownClick: PropTypes.func,
	onFocus: PropTypes.func,
	onKeyPress: PropTypes.func,
	onKeyUp: PropTypes.func,
	onMouseDown: PropTypes.func,
	onSelect: PropTypes.func,
	onUnselect: PropTypes.func,
	placeholder: PropTypes.string,
	readonly: PropTypes.bool,
	required: PropTypes.bool,
	scrollHeight: PropTypes.string,
	selectedItemTemplate: PropTypes.func,
	size: PropTypes.number,
	style: PropTypes.object,
	suggestions: PropTypes.array,
	tabindex: PropTypes.number,
	tooltip: PropTypes.string,
	tooltipOptions: PropTypes.object,
	type: PropTypes.string,
	value: PropTypes.any,
};

export default AutoComplete;
